// CSV data URL
const csvUrl = 'https://raw.githubusercontent.com/balajimandava/hospital-csv/main/Hospital%20ER.csv';

// Fetching CSV data
d3.csv(csvUrl)
  .then((data) => {
    // Transforming data into hierarchical structure
    const treeData = transformData(data);

    // Rendering the tree graph
    renderTreeGraph(treeData);
  })
  .catch((error) => {
    console.error('Error fetching data:', error);
  });

// Function to transform data into hierarchical structure
function transformData(data) {
  const root = { name: 'Hospital ER Data', children: [] };

  // Grouping data by date
  const dates = {};
  data.forEach((d) => {
    const date = d.date;
    if (!dates[date]) {
      dates[date] = { name: date, children: [] };
      root.children.push(dates[date]);
    }

    // Grouping data by patient ID
    const patientId = d.patient_id;
    if (!dates[date][patientId]) {
      dates[date][patientId] = { name: patientId, children: [] };
      dates[date].children.push(dates[date][patientId]);
    }

    // Grouping data by patient gender
    const patientGender = d.patient_gender;
    if (!dates[date][patientId][patientGender]) {
      dates[date][patientId][patientGender] = { name: patientGender, children: [] };
      dates[date][patientId].children.push(dates[date][patientId][patientGender]);
    }

    // Grouping data by patient age
    const patientAge = d.patient_age;
    if (!dates[date][patientId][patientGender][patientAge]) {
      dates[date][patientId][patientGender][patientAge] = { name: patientAge, children: [] };
      dates[date][patientId][patientGender].children.push(dates[date][patientId][patientGender][patientAge]);
    }

    // Grouping data by patient satisfaction score
    const patientSatScore = d.patient_sat_score;
    if (!dates[date][patientId][patientGender][patientAge][patientSatScore]) {
      dates[date][patientId][patientGender][patientAge][patientSatScore] = { name: patientSatScore, children: [] };
      dates[date][patientId][patientGender][patientAge].children.push(dates[date][patientId][patientGender][patientAge][patientSatScore]);
    }

    // Grouping data by patient name initials
    const patientNameInitials = d.patient_first_inital + d.patient_last_name;
    if (!dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials]) {
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials] = { name: patientNameInitials, children: [] };
      dates[date][patientId][patientGender][patientAge][patientSatScore].children.push(dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials]);
    }

    // Grouping data by patient race
    const patientRace = d.patient_race;
    if (!dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace]) {
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace] = { name: patientRace, children: [] };
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials].children.push(dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace]);
    }

    // Grouping data by patient admin flag
    const patientAdminFlag = d.patient_admin_flag;
    if (!dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag]) {
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag] = { name: patientAdminFlag, children: [] };
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace].children.push(dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag]);
    }

    // Grouping data by patient wait time
    const patientWaitTime = d.patient_waittime;
    if (!dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag][patientWaitTime]) {
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag][patientWaitTime] = { name: patientWaitTime };
      dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag].children.push(dates[date][patientId][patientGender][patientAge][patientSatScore][patientNameInitials][patientRace][patientAdminFlag][patientWaitTime]);
    }
  });

  return root;
}

// Function to render the tree graph
function renderTreeGraph(data) {
  // Set the dimensions and margins of the diagram
  const margin = {
    top: 20,
    right: 90,
    bottom: 20,
    left: 90,
  };
  const width = 960 - margin.left - margin.right;
  const height = 500 - margin.top - margin.bottom;

  // Append the SVG object to the body of the page
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.right + margin.left)
    .attr('height', height + margin.top + margin.bottom)
    .append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);

  // Create a tree layout with specified size
  const tree = d3.tree().size([height, width]);
  const root = d3.hierarchy(data);
  const treeData = tree(root);

  // Extract nodes and links
  const nodes = treeData.descendants();
  const links = treeData.links();

  // Create links
  svg.selectAll('path.link')
    .data(links)
    .enter()
    .append('path')
    .attr('class', 'link')
    .attr('d', d3.linkHorizontal()
      .x(d => d.y)
      .y(d => d.x));

  // Create nodes
  const node = svg.selectAll('g.node')
    .data(nodes)
    .enter()
    .append('g')
    .attr('class', 'node')
    .attr('transform', d => `translate(${d.y},${d.x})`);

  // Add circles for nodes
  node.append('circle')
    .attr('r', 5
